//#######################################################################
//#
//#	Copyright 	Earth People Technology Inc. 2016
//#
//#
//# File Name:  EPT_4CE6_AF_D1_Top.v
//# Author:     Earth People Technology
//# Date:       July 26, 2016
//# Revision:   A
//#
//# Development: EPT I2C Master Project 
//# Application: Altera Cyclone IV FPGA
//# Description: This file contains verilog code which will allow access
//#              to Active Transfer Library. 
//#               
//#              
//#
//#************************************************************************
//#
//# Revision History:	
//#			DATE		VERSION		DETAILS		
//#			7/26/16 	1			Created
//#
//#									
//#
//#######################################################################

`include "../src/dso_define.v"


//************************************************************************
//* Module Declaration
//************************************************************************

module EPT_4CE6_AF_D1_Top (

	
	input  wire [1:0]           aa,
	input  wire [1:0]           bc_in,
	output wire [2:0]           bc_out,
	inout  wire [7:0]		    bd_inout,
	
	output wire   [7:0]        XIO_1,       //XIO -- D2-D9
	output wire   [7:0]        XIO_2,       //XIO -- D10-D12
	output wire   [7:0]        XIO_3,       //XIO -- D22-D29
	//output wire   [7:0]        XIO_4,       //XIO -- D30-D37
	//output wire   [7:0]        XIO_5,       //XIO -- D38-D45
	//output wire   [7:0]        XIO_6,       //XIO -- D46-D53
	//input wire    [5:0]        XIO_7,       //XIO -- D69,D70,D71,D74,D75,D76
	
	//PGA SPI Bus Communications
	output wire                PGA_MOSI,
	output wire                PGA_CLK,
	input wire                 PGA_MISO,
	output wire                PGA_CS,
	output wire                PGA_ENABLE,
	
    // ADC Interface	
	output reg                 ADC_ENCODE,
	input wire   [7:0]         ADC_BUS,
	
	
	inout wire                  I2C_SDA_0,
	input wire                  I2C_SCL_0,
	
	//Transceiver Control Signals
	output wire                TR_DIR_1,
	output wire                TR_OE_1,
	
	output wire                TR_DIR_2,
	output wire                TR_OE_2,

	output wire                TR_DIR_3,
	output wire                TR_OE_3

	);

   //-----------------------------------------------
   // Parameters
   //-----------------------------------------------
    //State Machine 
   parameter 	IDLE                        = 0,
				START_ADC_CONV              = 1,
				ENCODE_HIGH                 = 2,
                ENCODE_LOW                  = 3,
                CHECK_FOR_MAX_SAMPLES       = 4,
				UPDATE_CH_SELECT            = 5,
                TRANSFER_TO_HOST            = 6,
                TRANSFER_COMPLETE           = 7;
				
				
	//PGA Commands
	parameter   PGA_WRITE_CMD               = 4'h8;
	parameter   PGA_READ_CMD                = 4'h9;
	parameter   PGA_CHANNEL_1               = 4'h1;
	parameter   PGA_CHANNEL_2               = 4'h2;  
	parameter   PGA_CHANNEL_3               = 4'h3; 	
	parameter   PGA_CHANNEL_4               = 4'h4; 	
	
	
    //Maximum Number of Samples to collect in mem_arry
	parameter   MAX_SAMPLES                 = 16'hff;
	
	//Maximum Number of Frames to send to the PC
	parameter   ADC_MAX_FRAME_COUNT         = 8'h20;
	
	
`ifdef SIM
   	reg [8*30:1] state_name;
`endif
   
 
//***************************************************************************
//* Internal Signals and Registers Declarations
//***************************************************************************
   	reg [7:0]                   state, next;

    wire                       CLK_66;
	reg                        RST;

	wire [23:0]                UC_IN;
	wire [21:0]                UC_OUT;
	
	//Trigger Signals
	wire [7:0]                  trigger_out;
	wire [7:0]                  trigger_in_byte;	
	
	//Transfer registers
	wire                        transfer_1_start;
	wire                        transfer_receive_busy;
	wire                        transfer_1_receive_in;
	wire [7:0]                  transfer_1_address;
	reg                         serial_receive_start_transfer_reg;

	reg                         transfer_2_start;
	wire                        transfer_2_receive_in;
	wire   [7:0]                transfer_2_data_in;
	wire   [7:0]                transfer_2_data_out;
	wire   [7:0]                transfer_2_data;
	
	reg                         transfer_3_start;
	wire                        transfer_3_receive_in;
	wire   [7:0]                transfer_3_control_register_number;
	
	reg                         transfer_4_start;
	wire                        transfer_4_receive_in;
	wire   [7:0]                transfer_4_control_register_contents;

    //Transfer Control registers
	//wire   [7:0]                control_register;
	
	//Serial SPI Receive Registers Unit 1
	wire  [7:0]                 serial_receive_out_byte;
	reg                         master_spi_initiate_receive;
	wire                        spi_byte_received;
	
	//Receive Storage registers Unit 1
    reg  [1:0]                  serial_receive_data_reg;

	//Serial SPI Transmit Registers
	wire                        spi_transmit_complete;
	
	//channel_selected_transmit_register
	reg   [7:0]                 channel_selected_transmit_register;	
	reg                         channel_selected_spi_transmit_start;
	
	//SPI Command registers
	reg   [7:0]                 spi_command_register;
	reg                         spi_command_start; 
	
	//SPI Command Multiplexor
	reg   [1:0]                 spi_write_control_reg;
	reg   [1:0]                 spi_read_control_reg;
	reg   [1:0]                 channel_selected_spi_transmit_start_reg;
	
	//I2C Bus 0 registers
	reg   [7:0]                 i2c_0_command_register;
	reg                         i2c_0_command_start;
	reg                         i2c_0_command_start_reg;
	wire                        i2c_serial_clk;
	
		
	//Block 1 Control Registers
	reg                         start_block_out;
	wire                        block_byte_ready;
	wire                        block_in_rcv;
	wire  [7:0]                 uc_length;
	wire                        block_busy;
	wire  [7:0]                 block_out_byte;

	
	
	//ADC conversion start control registers
	reg   [1:0]                 adc_cnvst_reg;
	reg                         adc_convst_cmd;
	reg                         adc_convst_cmd_reg;
	reg  [23:0]                 adc_convst_cmd_count;
	wire [23:0]                 adc_convst_delay_value;
	reg  [7:0]                  adc_convst_frame_count;
	reg  [7:0]                  adc_convst_delay_command_register;
	reg                         adc_convst_delay_command_start;
	reg                         adc_convst_delay_command_reg;
	
	
	//ADC Channels To Read
	reg   [3:0]                 adc_channels_to_read;
	wire                        adc_scan_channel_1;
	wire                        adc_scan_channel_2;
	wire                        adc_scan_channel_3;
	wire                        adc_scan_channel_4;
	reg   [2:0]                 adc_channel_select;
	reg   [2:0]                 receive_endterm_select;
	
	//ADC Encode registers
	reg                         encode_high_reg;
	reg                         encode_low_reg;

    //Register to store the active channels
	reg   [3:0]                 selected_channel;
	
	//Block Memory Registers
    reg   [7:0]                 mem_array [255:0];
	reg   [7:0]                 mem_array_index;
	reg   [15:0]                mem_array_storage_index;
	reg   [1:0]                 adc_channel_select_state;
	reg                         adc_channel_select_complete;
	reg                         block_mem_array_count_max_reached;
	
	//Registers to maintain the samples in mem_array
	reg                         max_samples_reached_reg;

	//PGA Registers
	reg   [3:0]                 channel_1_gain_setting;	
	reg   [3:0]                 channel_2_gain_setting;	
	reg   [3:0]                 channel_3_gain_setting;	
	reg   [3:0]                 channel_4_gain_setting;	
	
	//Timer Tick Main Counter
	reg   [18:0]                 timer_tick_counter;
	
	//Host Interface registers
	reg                         host_oe;
	reg                         host_rd_latch;
	reg                         host_wr_latch;
	reg   [3:0]                 host_rd_latch_state;
	reg   [3:0]                 host_wr_latch_state;
	
	//Memory Interface registers
	wire   [7:0]                local_memory_address;
	wire   [7:0]                local_memory_data;
	wire                        local_memory_wr_latch;
	wire                        local_memory_rd_latch;
	wire                        local_memory_oe;
	wire                        adc_convst_delay_local_memory_wr_latch;
	wire                        spi_local_memory_wr_latch;
	wire                        i2c_0_local_memory_wr_latch;
	wire                        adc_convst_delay_local_memory_rd_latch;
	wire                        spi_local_memory_rd_latch;
	wire                        i2c_0_local_memory_rd_latch;
	wire                        adc_convst_delay_local_memory_oe;
	wire                        spi_local_memory_oe;
	wire                        i2c_0_local_memory_oe;
	
	//UC Control Register
	wire   [7:0]                adc_control_reg;	
	wire   [7:0]                spi_control_reg;
	wire   [7:0]                i2c_0_control_reg;
	wire   [7:0]                i2c_1_control_reg;
	wire   [7:0]                adc_convst_control_reg;
	wire   [7:0]                reset_control_reg;
	wire                        request_local_memory;
	wire                        adc_convst_delay_request_memory;
	wire                        spi_comms_request_memory;	
	wire                        grant_local_memory;
	wire                        i2c_0_comms_request_memory;
	
	//Reset Control
	reg                         reset_reg;


	//TEST ONLY
	wire                        transfer_busy;
	reg   [3:0]                 STATE_OUT;
	wire  [3:0]                 uc_controller_memory_state;
	wire  [3:0]                 ft245_test_state_out;
	wire   [7:0]                active_transfer_test_out;

	
	


   //------------------------------------------------
   // 	Signal Assignments	
   //------------------------------------------------
   
   assign            TR_DIR_1  = 1'b0; //1 = A to B; 0 = B to A
   assign            TR_OE_1  = 1'b0;
   assign            TR_DIR_2  = 1'b0; //1 = A to B; 0 = B to A
   assign            TR_OE_2  = 1'b0;
   assign            TR_DIR_3  = 1'b0; //1 = A to B; 0 = B to A
   assign            TR_OE_3  = 1'b1;  
   
   //PGA Enable
   assign            PGA_ENABLE = 1'b1;

   //Clock and Reset
   assign            CLK_66 = aa[1];
   //assign            RST = aa[0];
   
   //Local Memory Request/Grant registers
   assign            request_local_memory = adc_convst_delay_request_memory | spi_comms_request_memory | i2c_0_comms_request_memory;
   
    //Control Register Map
   assign            adc_scan_channel_1    = adc_control_reg[4];
   assign            adc_scan_channel_2    = adc_control_reg[5];
   assign            adc_scan_channel_3    = adc_control_reg[6];
   assign            adc_scan_channel_4    = adc_control_reg[7];

   //Block to host byte
   assign            block_out_byte = mem_array[mem_array_index];
   assign            uc_length = 8'h8;
   
   //Stub the transfer start for Transfer Endterm 1
   //assign            transfer_1_start = 1'b0;
   
   //Host Interface data bus
   assign             transfer_2_data = !host_oe ? transfer_2_data_in : 8'hzz;
   assign             transfer_2_data_out = host_oe ? transfer_2_data : 8'h00;
   
   //Local Memory Write Latch register
   assign             local_memory_wr_latch = spi_local_memory_wr_latch | adc_convst_delay_local_memory_wr_latch | i2c_0_local_memory_wr_latch;
   assign             local_memory_oe = spi_local_memory_oe | adc_convst_delay_local_memory_oe | i2c_0_local_memory_oe;
   assign             local_memory_rd_latch = spi_local_memory_rd_latch | adc_convst_delay_local_memory_rd_latch | i2c_0_local_memory_rd_latch;
 
	//Debug XIO_1 Bus
   //assign           XIO_1 = (local_memory_wr_latch | host_wr_latch) ? (local_memory_data | transfer_2_data) : (local_memory_address | transfer_1_address);
    //assign           XIO_1 = host_wr_latch ? transfer_2_data : transfer_1_address;
//   assign           XIO_1 = transfer_4_receive_in ? transfer_4_control_register_contents  : transfer_3_control_register_number;
    assign           XIO_1[0] = PGA_CS;       //LB0 --J10 Pin1
    assign           XIO_1[1] = PGA_CLK;      //LB1 --J10 Pin2
    assign           XIO_1[2] = PGA_MOSI;     //LB2 --J10 Pin3
    assign           XIO_1[3] = 1'b0;         //LB3 --J10 Pin4
    assign           XIO_1[4] = STATE_OUT[0]; //LB4 --J10 Pin6
    assign           XIO_1[5] = STATE_OUT[1]; //LB5 --J10 Pin7
    assign           XIO_1[6] = STATE_OUT[2]; //LB6 --J10 Pin8
    assign           XIO_1[7] = STATE_OUT[3]; //LB7 --J10 Pin9

	//Debug XIO_2 Bus
    assign           XIO_2[0] = uc_controller_memory_state[0];  //LB8  --J10 Pin10
    assign           XIO_2[1] = uc_controller_memory_state[1];  //LB9  --J9 Pin1
    assign           XIO_2[2] = uc_controller_memory_state[2];        //LB10 --J9 Pin2
    assign           XIO_2[3] = spi_comms_request_memory;     //LB11 --J9 Pin3
    assign           XIO_2[4] = grant_local_memory;           //LB12 --J9 Pin4
    assign           XIO_2[5] = host_wr_latch;                //LB13 --J9 Pin5
    assign           XIO_2[6] = host_rd_latch;                //LB14 --J9 Pin6
    assign           XIO_2[7] = 1'bz;                         //LB15 --J10 Pin5
	
	//Debug XIO_4 Bus
/*    assign           XIO_4[0] = 1'b0;//active_transfer_test_out[0];//block_out_byte[0];
    assign           XIO_4[1] = 1'b0;//active_transfer_test_out[1];//block_out_byte[1];
    assign           XIO_4[2] = 1'b0;//active_transfer_test_out[2];//block_out_byte[2];
    assign           XIO_4[3] = 1'b0;//active_transfer_test_out[3];//block_out_byte[3];
    assign           XIO_4[4] = 1'b0;//active_transfer_test_out[4];//block_out_byte[4];
    assign           XIO_4[5] = 1'b0;//active_transfer_test_out[5];//block_out_byte[5];
    assign           XIO_4[6] = 1'b0;//active_transfer_test_out[6];//block_out_byte[6];
    assign           XIO_4[7] = 1'b0;//active_transfer_test_out[7];//block_out_byte[7];
	
	
	//Debug XIO_5 Bus
    assign           XIO_5[0] = 1'b0;//mem_array_index[0];
    assign           XIO_5[1] = 1'b0;//mem_array_index[1];
    assign           XIO_5[2] = 1'b0;//mem_array_index[2];
    assign           XIO_5[3] = 1'b0;//mem_array_index[3];
    assign           XIO_5[4] = 1'b0;//mem_array_index[4];
    assign           XIO_5[5] = 1'b0;//block_byte_ready;
    assign           XIO_5[6] = 1'b0;
    assign           XIO_5[7] = 1'b0;
*/	
	//Debug XIO_3
    assign           XIO_3[0] = ADC_BUS[0];   //LB16 --J8 Pin1
    assign           XIO_3[1] = ADC_BUS[1];   //LB17 --J8 Pin2
    assign           XIO_3[2] = ADC_BUS[2];   //LB18 --J8 Pin3
    assign           XIO_3[3] = ADC_BUS[3];   //LB19 --J8 Pin4
    assign           XIO_3[4] = ADC_BUS[4];	  //LB20 --J8 Pin5
    assign           XIO_3[5] = ADC_BUS[5];   //LB21 --J8 Pin6
    assign           XIO_3[6] = ADC_BUS[6];   //LB22 --J8 Pin7
    assign           XIO_3[7] = ADC_BUS[7];   //LB23 --J8 Pin8

   //-----------------------------------------------
   // Create a State Test Output Signal
   //-----------------------------------------------
   always @(posedge CLK_66 or negedge RST)
     begin
	if (!RST)
	  STATE_OUT <= 0;
	else
	  begin
	     if (state[IDLE])
           STATE_OUT <= 4'h0;
	     if (state[START_ADC_CONV])
	       STATE_OUT <= 4'h1;
	     if (state[ENCODE_HIGH])
	       STATE_OUT <= 4'h2;
	     if (state[ENCODE_LOW])
	       STATE_OUT <= 4'h3;
	     if (state[CHECK_FOR_MAX_SAMPLES])
	       STATE_OUT <= 4'h4;
	     if (state[UPDATE_CH_SELECT])
	       STATE_OUT <= 4'h5;
	     if (state[TRANSFER_TO_HOST])
	       STATE_OUT <= 4'h6;
	     if (state[TRANSFER_COMPLETE])
	       STATE_OUT <= 4'h7;
	  end // else: !if(!RST_N)
     end // always @ (posedge CLK or negedge RST_N)

   //-----------------------------------------------
   // Store the register value to transmit to the PGA.
   //-----------------------------------------------
	always @(posedge CLK_66 or negedge aa[0])
	begin
	  if(!aa[0])
	  begin
			RST <= 1'b0;
			reset_reg <= 1'b0;
	  end
	  else
	  begin
	     if(reset_control_reg[0] & !reset_reg)
			begin
			   RST <= 1'b0;
		       reset_reg <= 1'b1;
			end
	     else if(reset_control_reg[0] & reset_reg)
		 begin
		    if(timer_tick_counter[14])
			begin
			   RST <= 1'b1;
			end
		 end
	     else if(!reset_control_reg[0] & reset_reg)
		 begin
			   reset_reg <= 1'b0;
		 end
		 else
		    RST <= 1'b1;
	  end
	end  


   
   //-----------------------------------------------
   // SPI Command register Multiplexor
   // This section will multiplex the SPI write and 
   // read commands into the spi_command_register and
   // spi_command_start registers.
   //-----------------------------------------------
	always @(posedge CLK_66 or negedge RST)
	begin
	  if(!RST)
	  begin
         spi_command_register <= 0;
		 spi_command_start <= 1'b0;
		 spi_write_control_reg <= 2'b00;
		 spi_read_control_reg <= 2'b00;
		 channel_selected_spi_transmit_start_reg <= 2'b00;
	  end
	  else
	  begin
	     //Write to SPI(PGA) from Host
	     if(spi_control_reg[3:0] & (spi_write_control_reg == 2'b00) & 
		    (spi_read_control_reg[1] == 1'b0) & (channel_selected_spi_transmit_start_reg[1] == 1'b0))
	     begin
		    spi_write_control_reg <= 2'b01;
			spi_command_register[7:4] <= PGA_WRITE_CMD;
				 case(spi_control_reg[3:0])
				 4'h1:
				     spi_command_register[3:0] <= PGA_CHANNEL_1;
				 4'h2:
				     spi_command_register[3:0] <= PGA_CHANNEL_2;
				 4'h3:
				     spi_command_register[3:0] <= PGA_CHANNEL_3;
				 4'h4:
				     spi_command_register[3:0] <= PGA_CHANNEL_4;
				 endcase
		 end
		 else if(spi_control_reg[3:0] & (spi_write_control_reg == 2'b01))
		 begin
		    spi_command_start <= 1'b1;
		    spi_write_control_reg <= 2'b10;		    
		 end
		 else if((spi_write_control_reg == 2'b10))
		 begin
		    spi_command_start <= 1'b0;
            if(spi_transmit_complete)
		       spi_write_control_reg <= 2'b11;		    
		 end
		 else if((spi_control_reg[3:0] == 4'h0) & (spi_write_control_reg == 2'b11))
		 begin
		    spi_write_control_reg <= 2'b00;		    
		 end

	     //Write to SPI(PGA) from the ADC FSM
	     if(channel_selected_spi_transmit_start & (channel_selected_spi_transmit_start_reg == 2'b00) & 
		    (spi_read_control_reg[1] == 1'b0) & (spi_write_control_reg[1] == 1'b0))
	     begin
		    channel_selected_spi_transmit_start_reg <= 2'b01;
			spi_command_register <= channel_selected_transmit_register;
		 end
		 else if((channel_selected_spi_transmit_start_reg == 2'b01))
		 begin
		    spi_command_start <= 1'b1;
		    channel_selected_spi_transmit_start_reg <= 2'b10;		    
		 end
		 else if((channel_selected_spi_transmit_start_reg == 2'b10))
		 begin
		    spi_command_start <= 1'b0;
            if(spi_transmit_complete)
		       channel_selected_spi_transmit_start_reg <= 2'b11;		    
		 end
		 else if(!channel_selected_spi_transmit_start & (channel_selected_spi_transmit_start_reg == 2'b11))
		 begin
		    channel_selected_spi_transmit_start_reg <= 2'b00;		    
		 end

	     //Read from SPI(PGA) and store in Local Memory
	     if(spi_control_reg[7:4] & (spi_read_control_reg == 2'b00) & 
		    (spi_write_control_reg[1] == 1'b0) & (channel_selected_spi_transmit_start_reg[1] == 1'b0))
	     begin
		    spi_read_control_reg <= 2'b01;
			spi_command_register[7:4] <= PGA_READ_CMD;
				 case(spi_control_reg[7:4])
				 4'h1:
				     spi_command_register[3:0] <= PGA_CHANNEL_1;
				 4'h2:
				     spi_command_register[3:0] <= PGA_CHANNEL_2;
				 4'h3:
				     spi_command_register[3:0] <= PGA_CHANNEL_3;
				 4'h4:
				     spi_command_register[3:0] <= PGA_CHANNEL_4;
				 endcase
		 end
		 else if(spi_control_reg[7:4] & (spi_read_control_reg == 2'b01))
		 begin
		    spi_command_start <= 1'b1;
		    spi_read_control_reg <= 2'b10;		    
		 end
		 else if((spi_read_control_reg == 2'b10))
		 begin
		    spi_command_start <= 1'b0;
            if(spi_transmit_complete)
		       spi_read_control_reg <= 2'b11;		    
		 end
		 else if((spi_control_reg[7:4] == 4'h0) & (spi_read_control_reg == 2'b11))
		 begin
		    spi_read_control_reg <= 2'b00;		    
		 end

 	 end
    end	  

   //-----------------------------------------------
   // Gain settings for the Analog Signal Conditioning 
   // circuit. Uses I2C Bus 0. Each channel address, 
   // command and data are written into uc_controller
   // memory then read by the I2C module.
   //-----------------------------------------------
	always @(posedge CLK_66 or negedge RST)
	begin
	  if(!RST)
	  begin
            i2c_0_command_register <= 0;
			i2c_0_command_start <= 1'b0;
			i2c_0_command_start_reg <= 1'b0;
	  end
	  else
	  begin
	      if(i2c_0_control_reg[7] & !i2c_0_command_start_reg)
	      begin
		    i2c_0_command_register <= i2c_0_control_reg;
			i2c_0_command_start <= 1'b1;
			i2c_0_command_start_reg <= 1'b1;
	      end
	      else if(i2c_0_control_reg[7] & i2c_0_command_start_reg)
	      begin
			i2c_0_command_start <= 1'b0;
			i2c_0_command_start_reg <= 1'b1;
	      end
	      else if(!i2c_0_control_reg[7] & i2c_0_command_start_reg)
	      begin
			i2c_0_command_start <= 1'b0;
			i2c_0_command_start_reg <= 1'b0;
	      end
	  end
	end  

   //-----------------------------------------------
   // Timer Tick Main Counter.
   //-----------------------------------------------
	always @(posedge CLK_66 or negedge aa[0])
	begin
	  if(!aa[0])
	  begin
			timer_tick_counter <= 0;
	  end
	  else
	  begin
	      if(trigger_in_byte)
	      begin
             timer_tick_counter <= 0;
          end
		  else
		  begin
			timer_tick_counter <= timer_tick_counter + 1'd1;
	      end
	  end
	end  

   //-----------------------------------------------
   // Read Latch and Output Enable register decode.
   //-----------------------------------------------
	always @(posedge CLK_66 or negedge RST)
	begin
	  if(!RST)
	  begin
			host_rd_latch <= 1'b0;
			host_oe <= 1'b0;
			transfer_2_start <= 1'b0;
			host_rd_latch_state <= 2'b00;
	  end
	  else
	  begin
         case(host_rd_latch_state)
         2'b00:
         begin
	        if(trigger_in_byte[2])
			begin
			   host_oe <= 1'b1;
			   host_rd_latch_state <= 2'b01;
			end
         end
         2'b01:
         begin
	        if(transfer_1_receive_in)
	        begin
	           host_rd_latch <= 1'b0;
			   host_rd_latch_state <= 2'b10;
	        end
         end
         2'b10:
         begin
	        if(trigger_in_byte[1])
	        begin
	           host_rd_latch <= 1'b1;
			   transfer_2_start <= 1'b1;
			   host_rd_latch_state <= 2'b11;
	        end
         end
         2'b11:
         begin
		    if(timer_tick_counter[5])
	        begin
	           host_rd_latch <= 1'b0;
			   host_oe <= 1'b0;
			   transfer_2_start <= 1'b0;
			   host_rd_latch_state <= 2'b00;
	        end
         end
         endcase
	  end
	end  



   //-----------------------------------------------
   // Write Latch register decode.
   //-----------------------------------------------
	always @(posedge CLK_66 or negedge RST)
	begin
	  if(!RST)
	  begin
			host_wr_latch <= 1'b0;
			host_wr_latch_state <= 2'b00;
	  end
	  else
	  begin
         case(host_wr_latch_state)
         2'b00:
         begin
	        if(transfer_1_receive_in & !host_oe)
	        begin
	           host_wr_latch <= 1'b0;
			   host_wr_latch_state <= 2'b01;
	        end
         end
         2'b01:
         begin
	        if(transfer_2_receive_in)
	        begin
	           host_wr_latch <= 1'b0;
			   host_wr_latch_state <= 2'b10;
	        end
         end
         2'b10:
         begin
	        if(trigger_in_byte[0])
	        begin
	           host_wr_latch <= 1'b1;
			   host_wr_latch_state <= 2'b11;
	        end
         end
         2'b11:
         begin
		    if(timer_tick_counter[5])
	        begin
	           host_wr_latch <= 1'b0;
			   host_wr_latch_state <= 2'b00;
 	        end
        end
         endcase
	  end
	end  



   //-----------------------------------------------
   // Store the adc_convst_delay register 
   // Uses three separate trigger_in bits 
   // adc_convst_delay_command_start 
   // adc_convst_delay_command_register 
   // adc_convst_control_reg   
   //-----------------------------------------------
	always @(posedge CLK_66 or negedge RST)
	begin
	  if(!RST)
	  begin
            adc_convst_delay_command_register <= 0;
			adc_convst_delay_command_start <= 1'b0;
			adc_convst_delay_command_reg <= 1'b0;
	  end
	  else
	  begin
	      if(adc_convst_control_reg[0] & !adc_convst_delay_command_reg)
	      begin
		    adc_convst_delay_command_register <= `ADC_CONVST_DELAY_COMMAND;
			adc_convst_delay_command_start <= 1'b1;
			adc_convst_delay_command_reg <= 1'b1;
	      end
	      else if(adc_convst_control_reg[0] & adc_convst_delay_command_reg)
	      begin
			adc_convst_delay_command_start <= 1'b0;
			adc_convst_delay_command_reg <= 1'b1;
	      end
	      else if(!adc_convst_control_reg[0] & adc_convst_delay_command_reg)
	      begin
			adc_convst_delay_command_start <= 1'b0;
			adc_convst_delay_command_reg <= 1'b0;
	      end
	  end
	end  

   //-----------------------------------------------
   // ADC conversion start command from adc_control_reg[0]
   // Single conversion or repeated conversion are determined   
   //-----------------------------------------------
	always @(posedge CLK_66 or negedge RST)
	begin
	  if(!RST)
	  begin
	        ADC_ENCODE <= 1'b0;
	        adc_convst_cmd <= 1'b0;
			adc_convst_cmd_reg <= 1'b0;
			adc_convst_cmd_count <= 0;
			adc_convst_frame_count <= 0;
			adc_cnvst_reg <= 2'b00;
	  end
	  else
	  begin 
	      if(trigger_in_byte[3] & adc_control_reg[0] & !adc_convst_cmd_reg)
	      begin
		     adc_convst_cmd <= 1'b1;
			 adc_convst_cmd_reg <= 1'b1;
	         adc_convst_cmd_count <= 0;
			 adc_cnvst_reg <= 2'b00;
	      end
	      else if(adc_control_reg[3] & adc_convst_cmd_reg)
	      begin
		      adc_convst_cmd <= 1'b0;
	          adc_convst_cmd_reg <= 1'b0;
	          adc_convst_cmd_count <= 0;
			  adc_cnvst_reg <= 2'b00;
	      end
	      else if(adc_convst_cmd_reg)
	      begin
		      case(adc_cnvst_reg)
			  2'b00:
			  begin
			     if(state[CHECK_FOR_MAX_SAMPLES])
				    adc_cnvst_reg <= 2'b01;
		         else if(adc_convst_cmd_count < adc_convst_delay_value)
		         begin
		            adc_convst_cmd_count <= adc_convst_cmd_count + 1'd1;
		         end
				 else
		         begin
				    ADC_ENCODE <= !ADC_ENCODE;
					adc_convst_cmd_count <= 0;
		            //adc_convst_cmd <= 1'b1;
					//adc_cnvst_reg <= 2'b01;
		         end
			  end
			  2'b01:
			  begin
			    if(state[UPDATE_CH_SELECT])
			       adc_cnvst_reg <= 2'b00;
				else if(state[TRANSFER_TO_HOST])
			       adc_cnvst_reg <= 2'b10;
			  end
			  2'b10:
			  begin
			    if(state[TRANSFER_COMPLETE])
				begin
				   if(adc_convst_frame_count < ADC_MAX_FRAME_COUNT)
				   begin
                      adc_convst_frame_count <= adc_convst_frame_count + 1'd1;
				      adc_cnvst_reg <= 2'b00;
				   end
				   else
				      adc_cnvst_reg <= 2'b11;
				end
			  end
			  2'b11:
			  begin
		         adc_convst_cmd <= 1'b0;
			     adc_convst_frame_count <= 0;
			     adc_cnvst_reg <= 2'b00;
			     adc_convst_cmd_reg <= 1'b0;
              end
			  endcase
	      end
	  end
	end  
	
	//-----------------------------------------------
   // Register the ADC_Encode High and Low assertions   
   //-----------------------------------------------
	always @(posedge CLK_66 or negedge RST)
	begin
	  if(!RST)
	  begin
			encode_high_reg <= 1'b0;
			encode_low_reg <= 1'b0;
	  end
	  else
	  begin
	      if(state[UPDATE_CH_SELECT])
	      begin
			encode_high_reg <= 1'b0;
			encode_low_reg <= 1'b0;
	      end
	      else if(ADC_ENCODE)
	      begin
			    encode_high_reg <= 1'b1;
			    encode_low_reg <= 1'b0;
	      end
	      else if(!ADC_ENCODE)
	      begin
			    encode_high_reg <= 1'b0;
			    encode_low_reg <= 1'b1;
	      end

	  
	  /*	      if(state[ENCODE_HIGH])
	      begin
		     if(ADC_ENCODE)
			 begin
			    encode_high_reg <= 1'b1;
			    encode_low_reg <= 1'b0;
			 end
	      end
	      else if(state[ENCODE_LOW])
	      begin
		     if(ADC_ENCODE)
			 begin
			    encode_high_reg <= 1'b0;
			    encode_low_reg <= 1'b1;
			 end
	      end
	      else if(state[UPDATE_CH_SELECT])
	      begin
			encode_high_reg <= 1'b0;
			encode_low_reg <= 1'b0;
	      end
*/	  end
	end  

	

   //-----------------------------------------------
   // Store the ADC Channels to be scanned. 
   // 
   //-----------------------------------------------
/*	always @(posedge CLK_66 or negedge RST)
	begin
	  if(!RST)
	  begin
            selected_channel <= 4'b0000;
	  end
	  else
	  begin 
	      if(state[ENCODE_LOW] & (selected_channel == 4'b0000))
	      begin
	         selected_channel = {adc_scan_channel_4,adc_scan_channel_3,adc_scan_channel_2,adc_scan_channel_1};
	      end
	  end
	end  
*/
   //-----------------------------------------------
   // Select the next active channel to scan by the ADC.
   // Set the adc_channel_select equal to the adc_channels_to_read
   // register.    
   //-----------------------------------------------
	always @(posedge CLK_66 or negedge RST)
	begin
	  if(!RST)
	  begin
	        mem_array_storage_index <= 0;
			adc_channel_select_state <= 2'b00;
			adc_channel_select_complete <= 1'b0;
            channel_selected_spi_transmit_start <= 1'b0;
			channel_selected_transmit_register <= 0;
            selected_channel <= 4'b0000;
	  end
	  else
	  begin 
	      if(state[ENCODE_LOW] & (selected_channel == 4'b0000))
	      begin
	         selected_channel = {adc_scan_channel_4,adc_scan_channel_3,adc_scan_channel_2,adc_scan_channel_1};
	      end

          if(state[UPDATE_CH_SELECT])
	      begin
		      case(adc_channel_select_state)
			  2'b00:
			  begin
			       channel_selected_transmit_register[7:4] <= PGA_WRITE_CMD;//8'h2a;
				   channel_selected_spi_transmit_start <= 1'b0;
				   //if(spi_transmit_complete)
				      adc_channel_select_state <= 2'b01;
			  end
			  2'b01:
			  begin
			     casex(selected_channel)
			     4'bxxx1:
			     begin
			       channel_selected_transmit_register[3:0] <= PGA_CHANNEL_1;
				   selected_channel[0] <= 1'b0;
				   channel_selected_spi_transmit_start <= 1'b1;
			     end
			     4'bxx1x:
				 begin
			       channel_selected_transmit_register[3:0] <= PGA_CHANNEL_2;
				   selected_channel[1] <= 1'b0;
				   channel_selected_spi_transmit_start <= 1'b1;
				 end
			     4'bx1xx:
				 begin
			       channel_selected_transmit_register[3:0] <= PGA_CHANNEL_3;
				   selected_channel[2] <= 1'b0;
				   channel_selected_spi_transmit_start <= 1'b1;
				 end
			     4'b1xxx:
				 begin
			       channel_selected_transmit_register[3:0] <= PGA_CHANNEL_4;
				   selected_channel[3] <= 1'b0;
				   channel_selected_spi_transmit_start <= 1'b1;
				 end
			     endcase
				 adc_channel_select_state <= 2'b10;
			  end
			  2'b10:
			  begin
				 channel_selected_spi_transmit_start <= 1'b0;
				 if(spi_transmit_complete)
				    adc_channel_select_state <= 2'b11;
			  end
			  2'b11:
			  begin
				   channel_selected_spi_transmit_start <= 1'b0;
				 if(!spi_transmit_complete)
				   begin
				      if(mem_array_storage_index < 16'hff)
				         mem_array_storage_index <= mem_array_storage_index + 1'd1;
				      adc_channel_select_state <= 2'b00;
			          adc_channel_select_complete <= 1'b1;
				   end
              end
			  endcase
	      end
		  else if(state[TRANSFER_COMPLETE])
		  begin
	        mem_array_storage_index <= 0;
		  end
		  else
             adc_channel_select_complete <= 1'b0;
	  end
	end  
	
   //-----------------------------------------------
   // Store data from the read of the ADC into the 
   // memory array.
   //-----------------------------------------------
	always @(posedge CLK_66 or negedge RST)
	begin
	  if(!RST)
	  begin
			serial_receive_data_reg <= 2'b00;
			max_samples_reached_reg <= 1'b0;
	  end
	  else
	  begin
	     case(serial_receive_data_reg)
		    2'b00:
            if(state[ENCODE_HIGH])
			begin
               mem_array[mem_array_storage_index] <= ADC_BUS;
               serial_receive_data_reg <= 2'b01;
			end
			2'b01:
            if(state[ENCODE_LOW])
			begin
               serial_receive_data_reg <= 2'b10;
			end
			2'b10:
	        if(state[CHECK_FOR_MAX_SAMPLES])
			begin
			   if(mem_array_storage_index >= MAX_SAMPLES)
			   begin
			      max_samples_reached_reg <= 1'b1;
				  serial_receive_data_reg <= 2'b11;
			   end
			   else
			      serial_receive_data_reg <= 2'b11;
			end
			2'b11:
	        if(state[UPDATE_CH_SELECT] & !max_samples_reached_reg)
			begin
			   if((adc_channel_select_state == 2'b11) & !spi_transmit_complete)
			      serial_receive_data_reg <= 2'b00;
			end
			else if(state[TRANSFER_TO_HOST])
			begin
			      max_samples_reached_reg <= 1'b0;
			      serial_receive_data_reg <= 2'b00;
			end
	     endcase
	  end
	end
	

   //-----------------------------------------------
   // Start Transfer on the Active Block EndTerm 1
   // This will send the ADC Data to the EndTerm 1.
   //-----------------------------------------------
	always @(posedge CLK_66 or negedge RST)
	begin
	  if(!RST)
	  begin
			block_mem_array_count_max_reached <= 1'b0;
			start_block_out <= 1'b0;
			serial_receive_start_transfer_reg <= 1'b0;
			mem_array_index <= 0;
	  end
	  else
	  begin
         if(state[TRANSFER_TO_HOST] & !serial_receive_start_transfer_reg)
         begin
		     start_block_out <= 1'b1;
			 serial_receive_start_transfer_reg <= 1'b1;
			 
		 end
		 else if(state[TRANSFER_TO_HOST] & serial_receive_start_transfer_reg)
		 begin
		    if(block_byte_ready)
			begin
		       start_block_out <= 1'b0;
			   mem_array_index <= mem_array_index + 1'd1;
			end
			   
			if(mem_array_index == mem_array_storage_index)
			   block_mem_array_count_max_reached <= 1'b1;
		 end
		 else if(state[TRANSFER_COMPLETE])
         begin
			 block_mem_array_count_max_reached <= 1'b0;
		     start_block_out <= 1'b0;
			 serial_receive_start_transfer_reg <= 1'b0;
			 mem_array_index <= 0;
		 end
	  end
	end
	
   //--------------------------------------------------
   // Write State Machine
   //           IDLE                        = 0,
   //           START_ADC_CONV              = 1,
   //           ENCODE_HIGH                 = 2,
   //           ENCODE_LOW                  = 3,
   //           CHECK_FOR_MAX_SAMPLES       = 4,
   //           UPDATE_CH_SELECT            = 5,
   //           TRANSFER_TO_HOST            = 6,
   //           TRANSFER_COMPLETE           = 7;
   //--------------------------------------------------  
   // Next State Logic
   always @(posedge CLK_66 or negedge RST)
     begin
	if (!RST)
	  begin
	     state <= 8'h000;
	     state[IDLE] <= 1'b1;
	  end
	else
	  state <= next;
     end

	 always @ ( state or mem_array_index or block_byte_ready
	      or master_spi_initiate_receive or spi_byte_received 
	      or  adc_convst_cmd or adc_channel_select_complete
		  or encode_high_reg or encode_low_reg or adc_channel_select_state
		  or max_samples_reached_reg or serial_receive_start_transfer_reg
		  or spi_transmit_complete or block_mem_array_count_max_reached)

     begin
	next = 8'h000;

	if (state[IDLE])
	  begin
		 if(adc_convst_cmd)
	       next[START_ADC_CONV] = 1'b1;
		else
	       next[IDLE] = 1'b1;			
	  end

	if (state[START_ADC_CONV])
		    next[ENCODE_HIGH] = 1'b1;

	if (state[ENCODE_HIGH])
	begin
	   if(encode_high_reg)
          next[ENCODE_LOW] = 1'b1;
       else
	      next[ENCODE_HIGH] = 1'b1;
	end

	if (state[ENCODE_LOW])
	begin
	   if(encode_low_reg)
	      next[CHECK_FOR_MAX_SAMPLES] = 1'b1;
	    else 
		    next[ENCODE_LOW] = 1'b1;
	end

	if (state[CHECK_FOR_MAX_SAMPLES])
	begin
	   //if(mem_array_storage_index > MAX_SAMPLES)
	   if(max_samples_reached_reg)
          next[TRANSFER_TO_HOST] = 1'b1;
       else
          next[UPDATE_CH_SELECT] = 1'b1;
	end

	if (state[UPDATE_CH_SELECT])
	begin
	   if((adc_channel_select_state == 2'b10) & spi_transmit_complete)
		 next[ENCODE_HIGH] = 1'b1;
	   else
		 next[UPDATE_CH_SELECT] = 1'b1;
	end
	
	if (state[TRANSFER_TO_HOST])
	begin
	   if(block_mem_array_count_max_reached)
		 next[TRANSFER_COMPLETE] = 1'b1;
	   else
         next[TRANSFER_TO_HOST] = 1'b1;
	end

	if (state[TRANSFER_COMPLETE])
	begin
	   if(!serial_receive_start_transfer_reg)
	      next[TRANSFER_COMPLETE] = 1'b1;
	   else
			next[IDLE] = 1'b1;	
	end
	
`ifdef SIM
	if (state == (1 << IDLE))
	  state_name = "IDLE";
	else if (state == (1 << START_ADC_CONV))
	  state_name = "START_ADC_CONV";
	else if (state == (1 << ENCODE_HIGH))
	  state_name = "ENCODE_HIGH";
	else if (state == (1 << ENCODE_LOW))
	  state_name = "ENCODE_LOW";
	else if (state == (1 << CHECK_FOR_MAX_SAMPLES))
	  state_name = "CHECK_FOR_MAX_SAMPLES";
	else if (state == (1 <<   UPDATE_CH_SELECT))
	  state_name = "UPDATE_CH_SELECT";
	else if (state == (1 << TRANSFER_TO_HOST))
	  state_name = "TRANSFER_TO_HOST";
	else if (state == (1 << TRANSFER_COMPLETE))
	  state_name = "TRANSFER_COMPLETE";

	  
`endif
     end // always @ (...


   //-----------------------------------------------
   // Instantiate the SPI Interface
   //-----------------------------------------------
	spi_communications          SPI_COMMUNICATIONS_INST
	(
	.CLK                        (CLK_66), 
	.RST_N                      (RST),

	.SPI_DATA_OUT               (PGA_MOSI),
	.SPI_CLK                    (PGA_CLK),
	.SPI_DATA_IN                (PGA_MISO),
	.SPI_CS_N                   (PGA_CS),

	.MEMORY_ADDRESS             (local_memory_address),
	.MEMORY_DATA                (local_memory_data),
	.MEMORY_WRITE_LATCH         (spi_local_memory_wr_latch),
	.MEMORY_READ_LATCH          (spi_local_memory_rd_latch),
	.MEMORY_OUTPUT_ENABLE       (spi_local_memory_oe),
	
	.SPI_COMMAND                (spi_command_register),
	.SPI_COMMAND_LATCH          (spi_command_start),
	
	.SPI_TRANSMIT_COMPLETE      (spi_transmit_complete),
	.SPI_BYTE_RECEIVED          (spi_byte_received),
	
	.REQUEST_EN                 (spi_comms_request_memory),
	.GRANT_RDY                  (grant_local_memory)

	);

   //-----------------------------------------------
   // Instantiate the ADC CONVST DELAY Register
   //-----------------------------------------------

   adc_conversion_control      ADC_CONVST_DELAY_REG_INST
  (
   .CLK                        (CLK_66),
   .RST_N                      (RST),
   
   .ADC_CONVST_DELAY_REGISTER  (adc_convst_delay_value),
   
   .MEMORY_ADDRESS             (local_memory_address),
   .MEMORY_DATA                (local_memory_data),
   .MEMORY_WRITE_LATCH         (adc_convst_delay_local_memory_wr_latch),
   .MEMORY_READ_LATCH          (adc_convst_delay_local_memory_rd_latch),
   .MEMORY_OUTPUT_ENABLE       (adc_convst_delay_local_memory_oe),

   .ADC_CONVST_COMMAND         (adc_convst_delay_command_register),
   .ADC_CONVST_COMMAND_LATCH   (adc_convst_delay_command_start),

   .REQUEST_EN                 (adc_convst_delay_request_memory),
   .GRANT_RDY                  (grant_local_memory)
   
   );

   //-----------------------------------------------
   // I2C Slave SERDES Module 
   //-----------------------------------------------
    i2c_comms_interface         I2C_COMMS_INTERFACE_0_INST
	(
	.CLK                        (CLK_66),
	.RST_N                      (RST),
	
	.SERIAL_CLK                 (i2c_serial_clk),

	.I2C_DATA                   (I2C_SDA_0),
	.I2C_CLOCK                  (I2C_SCL_0),

	.MEMORY_ADDRESS             (local_memory_address),
	.MEMORY_DATA                (local_memory_data),
	.MEMORY_WRITE_LATCH         (i2c_0_local_memory_wr_latch),
	.MEMORY_READ_LATCH          (i2c_0_local_memory_rd_latch),
	.MEMORY_OUTPUT_ENABLE       (i2c_0_local_memory_oe),
	
	.I2C_COMMAND                (i2c_0_command_register),
	.I2C_COMMAND_LATCH          (i2c_0_command_start),
	
	.I2C_TRANSMIT_COMPLETE      (i2c_0_transmit_complete),
	.I2C_BYTE_RECEIVED          (i2c_0_byte_received),
	
	.REQUEST_EN                 (i2c_0_comms_request_memory),
	.GRANT_RDY                  (grant_local_memory)

     );

   //-----------------------------------------------
   // Instantiate the UC Controller
   //-----------------------------------------------
	uc_controller                               UC_CONTROLLER_INST
	(
	.CLK                                        (CLK_66), 
	.RST_N                                      (RST),

	.HOST_INTERFACE_ADDRESS                     (transfer_1_address),
	.HOST_INTERFACE_DATA                        (transfer_2_data),
	.HOST_INTERFACE_WRITE_LATCH                 (host_wr_latch),
	.HOST_INTERFACE_READ_LATCH                  (host_rd_latch),
	.HOST_INTERFACE_OUTPUT_ENABLE               (host_oe),

	.HOST_CONTROL_REG_NUMBER                    (transfer_3_control_register_number),
	.HOST_CONTROL_REG_CONTENTS                  (transfer_4_control_register_contents),
	.HOST_CONTROL_REG_NUMBER_LATCH              (transfer_3_receive_in),
	.HOST_CONTROL_REG_CONTENTS_LATCH            (transfer_4_receive_in),

	.CR_0                                       (adc_control_reg),
	.CR_1                                       (spi_control_reg),
	.CR_2                                       (i2c_0_control_reg),
	.CR_3                                       (i2c_1_control_reg),
	.CR_4                                       (adc_convst_control_reg),
	.CR_5                                       (reset_control_reg),
	.CR_6                                       (),
	.CR_7                                       (),
	
	.MEMORY_INTERFACE_ADDRESS                   (local_memory_address),
	.MEMORY_INTERFACE_DATA                      (local_memory_data),
	//.MEMORY_INTERFACE_WRITE_LATCH               (local_memory_wr_latch),
	.MEMORY_INTERFACE_WRITE_LATCH               (1'b0),
	.MEMORY_INTERFACE_READ_LATCH                (local_memory_rd_latch),
	.MEMORY_INTERFACE_OUTPUT_ENABLE             (local_memory_oe),

	.MEMORY_INTERFACE_EN                        (request_local_memory),
	.MEMORY_INTERFACE_RDY                       (grant_local_memory),
	
	.TIMEOUT_TICK                               (timer_tick_counter),

	.ERROR_OUT                                  (),

	.STATE_OUT                                  (uc_controller_memory_state)
	);

   //-----------------------------------------------
   // Instantiate the Serial Clock
   //-----------------------------------------------
	serial_clock               SERIAL_CLOCK_INST
	(	
	.CLK                       (CLK_66),
	.RST_N                     (RST),
	
	.SERIAL_CLK                (i2c_serial_clk)
	);

   //-----------------------------------------------
   // Instantiate the EPT Library
   //-----------------------------------------------

	active_transfer_library	   ACTIVE_TRANSFER_LIBRARY_INST
	(	
	.aa                        (aa),
	.bc_in                     (bc_in),
	.bc_out                    (bc_out),
	.bd_inout                  (bd_inout),

	.UC_IN                     (UC_IN),
	.UC_OUT                    (UC_OUT),
	
	.STATE_OUT                 (ft245_test_state_out),
	.TEST_OUT                  (active_transfer_test_out)
	
	);
	
   //-----------------------------------------------
   // Instantiate the EPT Active Modules
   //-----------------------------------------------
//`ifdef SIM
//wire [30*6-1:0]  uc_out_m;
//`else
wire [22*6-1:0]  uc_out_m;
//`endif	
eptWireOR # (.N(6)) wireOR (UC_OUT, uc_out_m);

    //
	// Trigger_In Mapping
	// trigger_in_byte[0] = Assert Write Latch.
	// trigger_in_byte[1] = Assert Read Latch.
	// trigger_in_byte[2] = Assert Output Enable.
	// trigger_in_byte[3] = Start ADC Conversions
	// trigger_in_byte[4] = Load adc_convst_delay register Byte 1
	// trigger_in_byte[5] = Load adc_convst_delay register Byte 2
	// trigger_in_byte[6] = Load adc_convst_delay register Byte 3
	// trigger_in_byte[7] = Master SPI Receive Start from the PGA
	active_trigger             ACTIVE_TRIGGER_INST
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
//`ifdef SIM
//	 .uc_out                   (uc_out_m[ 0*30 +: 30 ]),
//`else
	 .uc_out                   (uc_out_m[ 0*22 +: 22 ]),
//`endif	

	 .trigger_to_host          (trigger_out),
	 .trigger_to_device        (trigger_in_byte)
	
	);


	active_transfer            ACTIVE_TRANSFER_INST_1
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
//`ifdef SIM
//	 .uc_out                   (uc_out_m[ 1*30 +: 30 ]),
//`else
	 .uc_out                   (uc_out_m[ 1*22 +: 22 ]),
//`endif	
	
	 .start_transfer           (transfer_1_start),
	 .transfer_received        (transfer_1_receive_in),
	 
	 .transfer_busy            (transfer_receive_busy),
	
	 .uc_addr                  (3'h1),

	 .transfer_to_host         (),//serial_receive_data_storage),
	 .transfer_to_device       (transfer_1_address)//()	
	);

	active_transfer            ACTIVE_TRANSFER_INST_2
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
//`ifdef SIM
//	 .uc_out                   (uc_out_m[ 2*30 +: 30 ]),
//`else
	 .uc_out                   (uc_out_m[ 2*22 +: 22 ]),
//`endif	
	
	 .start_transfer           (transfer_2_start),
	 .transfer_received        (transfer_2_receive_in),
	
	 .transfer_busy            (),
	
	 .uc_addr                  (3'h2),

	 .transfer_to_host         (transfer_2_data_out),
	 .transfer_to_device       (transfer_2_data_in)	
	);
	
	active_transfer            ACTIVE_TRANSFER_INST_3
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
//`ifdef SIM
//	 .uc_out                   (uc_out_m[ 3*30 +: 30 ]),
//`else
	 .uc_out                   (uc_out_m[ 3*22 +: 22 ]),
//`endif	
	
	 .start_transfer           (transfer_3_start),
	 .transfer_received        (transfer_3_receive_in),

	 .transfer_busy            (),
	
	 .uc_addr                  (3'h3),

	 .transfer_to_host         (),
	 .transfer_to_device       (transfer_3_control_register_number)	
	);

	active_transfer            ACTIVE_TRANSFER_INST_4
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
//`ifdef SIM
//	 .uc_out                   (uc_out_m[ 4*30 +: 30 ]),
//`else
	 .uc_out                   (uc_out_m[ 4*22 +: 22 ]),
//`endif	
	
	 .start_transfer           (transfer_4_start),
	 .transfer_received        (transfer_4_receive_in),
	
	 .transfer_busy            (),
	
	 .uc_addr                  (3'h4),

	 .transfer_to_host         (),
	 .transfer_to_device       (transfer_4_control_register_contents)	
	);
	
	
	active_block               BLOCK_TRANSFER_INST
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
//`ifdef SIM
//	 .uc_out                   (uc_out_m[ 5*30 +: 30 ]),
//`else
	 .uc_out                   (uc_out_m[ 5*22 +: 22 ]),
//`endif	
	
	 .start_transfer           (start_block_out),
	 .transfer_received        (block_in_rcv),
	 
	 .transfer_ready           (block_byte_ready),
	 .transfer_busy            (block_busy),

	 .ept_length               (),
	
	 .uc_addr                  (3'h1),
	 .uc_length                (mem_array_storage_index),

	 .transfer_to_host         (block_out_byte),
	 .transfer_to_device       ()
	
	);


	
endmodule

