//#######################################################################
//#
//#	Copyright 	Earth People Technology Inc. 2016
//#
//#
//# File Name:  i2c_receive_model.v
//# Author:     Earth People Technology
//# Date:       July 26, 2016
//# Revision:   A
//#
//# Development: EPT I2C Slave Project 
//# Application: Altera Cyclone IV FPGA
//# Description: This file contains verilog code which will allow access
//#              to Active Transfer Library. 
//#               
//#              
//#
//#************************************************************************
//#
//# Revision History:	
//#			DATE		VERSION		DETAILS		
//#			7/26/16 	1			Created
//#
//#									
//#
//#######################################################################

`timescale 1ns/1ps

module i2c_receive_model(
   CLK, 
   RST_N,

   DATA_IN,
   SCLK,

   DOUT,
   DONE,
   START,

   PERFORM_READ,

   //TEST ONLY
   TEST_BUS
   );

   //----------------------------------------------
   // Parameter Declarations
   //----------------------------------------------
   parameter IDLE                  = 0,
             I2C_START             = 1,
	         RECEIVE_DATA          = 2,
             READ_DONE			   = 3,
             READ_DONE_DELAY	   = 4;

   //----------------------------------------------
   // Bit Count Parameter Declarations                        
   //----------------------------------------------
   parameter BIT_COUNT_IDLE         = 0,
			 BIT_COUNT_CLK_HI		= 1,
			 BIT_COUNT_CLK_LOW		= 2,
			 BIT_COUNT_INC_BYTE		= 3;
			 
   //--------------------------------------------------
   // IO Port Declarations
   //--------------------------------------------------     
   input wire                   CLK;
   input wire                   RST_N;
   input wire                   START;
   input wire                   DATA_IN;
   
   output reg [7:0]             DOUT;
   output  reg                  DONE;
   input wire                   SCLK;

   output  reg                  PERFORM_READ;
   
   output wire [3:0]            TEST_BUS;


   
   //------------------------------------------------
   // Wire and Register Declarations                
   //------------------------------------------------
   reg [9:0]                    state, next;

   // Bit Count Registers
	reg [3:0]                  in_bit_count;

	//Metastability registers
	reg                        i2c_clk_reg;

	// in bit counter
	reg [1:0]                  bit_count_state;
	
	// I2C Shift Registers
	reg [7:0]                  shift_in_reg;

	reg  [3:0]                   STATE_OUT;
   

//==================================================================
// Simulation Directives
//==================================================================
`ifdef SIM
   reg [8*20:1] state_name;
`endif

//=================================================================================
// Assignments	
//=================================================================================
   assign TEST_BUS = in_bit_count;
   
   
   //-----------------------------------------------
   // Create a State Test Output Signal
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  STATE_OUT <= 0;
	else
	  begin
	     if (state[IDLE])
           STATE_OUT <= 5'h0;
	     if (state[I2C_START])
	       STATE_OUT <= 5'h1;
	     if (state[RECEIVE_DATA])
	       STATE_OUT <= 5'h2;
	     if (state[READ_DONE])
	       STATE_OUT <= 5'h3;
	     if (state[READ_DONE_DELAY])
	       STATE_OUT <= 5'h4;
	  end // else: !if(!RST_N)
     end // always @ (posedge CLK or negedge RST_N)

	 //------------------------------------------------
   // Create a Read function in operation Signal 
   // This will inform the write_SPI_fsm.v to assert
   // DATA_OUT signal to a high.
   //------------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  PERFORM_READ <= 0;
	else
	  begin
	     if (state[IDLE])
	       PERFORM_READ <= 0;
	     else
	       PERFORM_READ <= 1;
	  end
     end // always @ (posedge CLK or negedge RST_N)

   //------------------------------------------------
   // Create a DONE Signal when the byte has been transmitted
   //------------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  DONE <= 0;
	else
	  begin
	     if (state[READ_DONE_DELAY])
	       DONE <= 1;
	     else
	       DONE <= 0;
	  end
     end // always @ (posedge CLK or negedge RST_N)

   //-----------------------------------------------
   // Register Block access
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
    begin
	if (!RST_N)
	begin
		DOUT <= 0;
	end
	else
    begin
        if(state[READ_DONE_DELAY])
        begin
                 DOUT <= shift_in_reg;
         end
    end
    end

   //-----------------------------------------------
   // Create a i2c_clk_reg signal. 
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	 i2c_clk_reg <= 0;
	else
	  begin
	     if ( SCLK )
	       i2c_clk_reg <= 1'b1;
	     else 
	       i2c_clk_reg <= 1'b0;
	  end
     end // always @ (posedge CLK or negedge RST_N)


   //------------------------------------------------
   // Create a bit_count of the number of bits received
   //------------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
	  in_bit_count <= 0;
	  bit_count_state <= BIT_COUNT_IDLE;
	end
	else
	  begin
	  
	     case( bit_count_state )
		     BIT_COUNT_IDLE:
			     if ( state[RECEIVE_DATA] )
				     bit_count_state <= BIT_COUNT_CLK_HI;
			 BIT_COUNT_CLK_HI:
			     if( i2c_clk_reg )
				 begin
				     bit_count_state <= BIT_COUNT_CLK_LOW;
	                 in_bit_count <= in_bit_count + 1;
				 end
			 BIT_COUNT_CLK_LOW:
			     if( !i2c_clk_reg )
				     bit_count_state <= BIT_COUNT_CLK_HI;
				 else if( in_bit_count >= 8'h8)
				     bit_count_state <= BIT_COUNT_INC_BYTE;
			 BIT_COUNT_INC_BYTE:
			     if( state[READ_DONE_DELAY] )
				 begin
				     bit_count_state <= BIT_COUNT_IDLE;
	                 in_bit_count <= 0;
				 end
			 default:
			 begin
				     bit_count_state <= BIT_COUNT_IDLE;
			 end
		 endcase
	  end
     end

   //------------------------------------------------
   // Shift Data In
   //------------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
			shift_in_reg <= 0;
	end
	else
	  begin
	     if((bit_count_state == BIT_COUNT_CLK_HI) & i2c_clk_reg)
	       begin
		  shift_in_reg[0] <= DATA_IN;
		  shift_in_reg[7:1] <= shift_in_reg[6:0];
	       end
	     else if (state[IDLE])
	       shift_in_reg <= 0;
	  end
     end // always @ (posedge CLK or negedge RST_N)
 
   //--------------------------------------------------
   // Write State Machine
   //--------------------------------------------------  
   // Next State Logic
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  begin
	     state <= 10'h00;
	     state[IDLE] <= 1'b1;
	  end
	else
	  state <= next;
     end

   always @ (START or in_bit_count or i2c_clk_reg or DATA_IN or state)
     begin
	next = 10'h00;

	if (state[IDLE])
	  begin
	     if (START)
	       next[I2C_START] = 1'b1;
	     else
	       next[IDLE] = 1'b1;
	  end

	if (state[I2C_START])
	  begin
	     if (!i2c_clk_reg)
	       next[RECEIVE_DATA] = 1'b1;
	     else
	       next[I2C_START] = 1'b1;
	  end

	  if (state[RECEIVE_DATA])
	  begin
	     if ((in_bit_count >= 8) & i2c_clk_reg)
	       next[READ_DONE_DELAY] = 1'b1;
	     else
	       next[RECEIVE_DATA] = 1'b1;
	  end
	  
	if (state[READ_DONE_DELAY])
    begin
        if (!i2c_clk_reg)
	       next[READ_DONE] = 1'b1;	
	     else
	       next[READ_DONE_DELAY] = 1'b1;
    end

	if (state[READ_DONE])
	  next[IDLE] = 1'b1;	

`ifdef SIM
	if (state == (1 << IDLE))
	  state_name = "IDLE";
	else if (state == (1 << I2C_START))
	  state_name = "I2C_START";
	else if (state == (1 << RECEIVE_DATA))
	  state_name = "RECEIVE_DATA";
	else if (state == (1 << READ_DONE))
	  state_name = "READ_DONE";	
	else if (state == (1 << READ_DONE_DELAY))
	  state_name = "READ_DONE_DELAY";	
`endif
     end // always @ (...

   
endmodule

   
