//==========================================================
//
//	Smartronix Proprietary Information
//
//
//	File Name:	iic_transmit_model.v
//	Author:	R. Jolly
//	Date:		October 14, 2008
//	Revision:	A
//
//
//	Development:	Environment Control Module UAV
//	Application:	Altera Stratix II
//	Description:	This file contains Verilog code that is designed
//				to encode and transmit I2C serial streams. 
//
//
//
//
//	Revision History:
//
//	DATE		VERSION			DETAILS
//	10/14/08		A		Created by R Jolly
//
//
//
//
//
//
//
//
//
//
//
//==================================================================
module iic_transmit_model (
			clk,
			reset,
			iic_data,
			iic_clock,
			holding_reg_empty,
			transmit_complete,
			data_in,
			data_valid,
			active

			);

    parameter log_width = 3;

    parameter    IDLE                   = 0,
                 READ_DATA_IN           = 1,
                 INITIATE_IIC_START     = 2,
                 LOAD_SHIFT_REG			= 3,
                 SHIFT_BIT_OUT			= 4,
                 CHECK_FOR_END_OF_WORD  = 5;



    input                       clk;
    input                       reset;

    output                      iic_data;
    output                      iic_clock;

	output                      holding_reg_empty;
    output                      transmit_complete;
    input	[(1<<log_width)-1:0]	data_in;
    input                       data_valid;
    input                       active;


  // IIC output signals
    reg				iic_data;
    reg				iic_clock;
	
	// Holding Registers
	reg      holding_reg_empty;

	wire					iic_data_pre;
	reg	[(1<<log_width)-1:0]	holding_reg;
	reg	[(1<<log_width)-1:0]	shift_reg;
	reg	[1:0]				phase;
	reg					iic_data_idle;
	reg					iic_data_select;
	reg					last_data_valid;
	
	reg  [7:0]   			bit_counter;

	// Transmit complete registers
	reg					transmit_complete;
	reg					transmit_complete_reg;



	assign	iic_data_pre = iic_data_select ? shift_reg[(1<<log_width)-1] :
							1'b0;
//							iic_data_idle;


//=================================================================================
//	Create the I2C output Data
//=================================================================================


	always @(negedge clk or negedge reset)
	begin
		if( !reset )
			iic_data = 1'b1;
		else
			iic_data = iic_data_pre;
	end

//=================================================================================
//	Create a shift out complete register
//=================================================================================

	always @(posedge clk or negedge reset)
	begin
		if (!reset)
		begin
				transmit_complete <= 1'b0;
				transmit_complete_reg <= 1'b0;
		end
		else
		begin
			if( !iic_data_idle & iic_data_select & !transmit_complete_reg )
			begin
				transmit_complete <= 1'b0;
				transmit_complete_reg <= 1'b1;
			end
			else if( !iic_data_idle & !iic_data_select & transmit_complete_reg )
			begin
				transmit_complete <= 1'b1;
				transmit_complete_reg <= 1'b1;
			end
			else if( iic_data_idle & !iic_data_select & transmit_complete_reg)
			begin
				transmit_complete <= 1'b0;
				transmit_complete_reg <= 1'b0;
			end
		end
	end


//=================================================================================
//	Create the I2C Bus Phases and shift registers
//=================================================================================

always @(posedge clk or negedge reset)
   if (!reset)
      begin
			iic_data_idle <= 1'b1;
			iic_data_select <= 1'b0;
			iic_clock <= 1'b0;
			last_data_valid <= 1'b0;
			holding_reg_empty <= 1'b1;
			phase <= 2'b00;
      end
   else
      begin
         case (phase)
            2'b00: begin
                     if (active)
                        begin
                           //bit_counter <= ((1<<log_width)-1);
                           bit_counter <= ((1<<log_width));
                          iic_data_idle <= 1'b0;
                           phase <= 2'b01;
                        end
                     else
                       iic_data_idle <= 1'b1;
                  end
            2'b01: begin
                     if (active)
                        begin
                           iic_clock <= 1'b0;
                           if (!holding_reg_empty)
                              begin
                                 shift_reg <= holding_reg;
                                 holding_reg_empty <= 1'b1;
                                 iic_data_select <= 1'b1;
                                 phase <= 2'b10;
                             end
                        end
                     else
                        begin
                           iic_clock <= 1'b1;
                           phase <= 2'b00;
                        end
                  end
            2'b10: begin
                     iic_clock <= !iic_clock;
                     if (iic_clock)
                        begin
                           if (bit_counter != 0)
                              shift_reg <= { shift_reg[((1<<log_width)-2):0], 1'b0 };
                           /*else if (!holding_reg_empty)
                              begin
                                 shift_reg <= holding_reg;
                                 holding_reg_empty <= 1'b1;
                              end*/
                           else
                              begin
                                 //phase <= 2'b01;
                                 phase <= 2'b11;
                                 iic_data_select <= 1'b0;
                              end
                           bit_counter <= bit_counter - 1;
                        end
                  end
            2'b11: begin
			          if(!active)
					  begin
                         iic_clock <= 1'b0;
                         phase <= 2'b00;
					  end
                    end
         endcase
           
         if (data_valid & !last_data_valid)
            begin
               holding_reg <= data_in;
               holding_reg_empty <= 1'b0;
            end
         last_data_valid <= data_valid;
      end

endmodule

